package com.javahonk;

import java.io.File;
import java.io.IOException;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;

import javax.mail.internet.MimeMessage;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.mail.MailException;
import org.springframework.mail.MailSender;
import org.springframework.mail.SimpleMailMessage;
import org.springframework.mail.javamail.JavaMailSender;
import org.springframework.mail.javamail.MimeMessageHelper;
import org.springframework.mail.javamail.MimeMessagePreparator;
import org.springframework.ui.freemarker.FreeMarkerTemplateUtils;

import freemarker.template.Configuration;
import freemarker.template.TemplateException;

public class MailHelper {

    private static final Logger logger = LogManager.getLogger(MailHelper.class.getName());

    @Autowired
    private MailSender mailSender;

    @Autowired
    @Qualifier("mailSender")
    private JavaMailSender javaMailSender;

    @Autowired
    Configuration freemarkerConfiguration;

    public void sendTemplatedMimeMessage(
            Map<String, Object> templatedMimeMessage,
            String template,
            String fromAddress,
            List<String> toAddresses,
            String mailSubject,
            Map<String, File> attachments) {

        MimeMessagePreparator preparator;
        try {
            preparator = new MimeMessagePreparator() {

                String messageText =
                        FreeMarkerTemplateUtils.processTemplateIntoString(
                                freemarkerConfiguration.getTemplate(template), templatedMimeMessage);

                @Override
                public void prepare(MimeMessage message) throws Exception {

                    MimeMessageHelper helper = new MimeMessageHelper(message, true);

                    for (String address : toAddresses) {
                        helper.addTo(address);
                    }

                    helper.setFrom(fromAddress);
                    helper.setSubject(mailSubject);

                    if (attachments != null) {

                        for (Entry<String, File> attachment : attachments.entrySet()) {

                            helper.addAttachment(attachment.getKey(), attachment.getValue());
                        }

                    }

                    helper.setText(messageText, true);
                }
            };

            this.javaMailSender.send(preparator);

        } catch (IOException | TemplateException | MailException ex) {

            logger.error("Failed to send email for templated message:" +
                    "\nFrom:" + fromAddress +
                    "\nTo:" + toAddresses +
                    "\nMessageMap:" + templatedMimeMessage, ex);

        }

    }

    public void sendMimeMessage(
            String mimeMessage,
            String fromAddress,
            List<String> toAddresses,
            String mailSubject,
            Map<String, File> attachments) {

        MimeMessagePreparator preparator = new MimeMessagePreparator() {

            @Override
            public void prepare(MimeMessage message) throws Exception {

                MimeMessageHelper helper = new MimeMessageHelper(message, true);

                for (String address : toAddresses) {
                    helper.addTo(address);
                }

                helper.setFrom(fromAddress);
                helper.setSubject(mailSubject);

                if (attachments != null) {

                    for (Entry<String, File> attachment : attachments.entrySet()) {

                        helper.addAttachment(attachment.getKey(), attachment.getValue());
                    }

                }

                helper.setText(mimeMessage);
            }
        };

        try {

            this.javaMailSender.send(preparator);

        } catch (MailException ex) {

            logger.error("Failed to send email for templated message:" +
                    "\nFrom:" + fromAddress +
                    "\nTo:" + toAddresses +
                    "\nMessage:" + mimeMessage, ex);
        }

    }

    public void sendSimpleMailMessage(
            SimpleMailMessage templateMessage,
            String message) {

        SimpleMailMessage msg = new SimpleMailMessage(templateMessage);

        msg.setText(message);

        try {

            this.mailSender.send(msg);

        } catch (MailException ex) {

            logger.error("Failed to send email for templated message:" +
                    "\nFrom:" + templateMessage.getFrom() +
                    "\nTo:" + templateMessage.getTo() +
                    "\nMessage:" + templateMessage.getText(), ex);

        }

    }

}
